@@ -25,11 +25,11 @@
 #ifndef NAUTILUS_UNDO_PRIVATE_H
 #define NAUTILUS_UNDO_PRIVATE_H
 
-#include "nautilus-distributed-undo.h"
-#include <gtk/gtkobject.h>
+#include <libnautilus/nautilus-distributed-undo.h>
+#include <glib-object.h>
 
-Nautilus_Undo_Manager nautilus_undo_get_undo_manager                (GtkObject             *attached_object);
-void                  nautilus_undo_attach_undo_manager             (GtkObject             *object,
-								     Nautilus_Undo_Manager  manager);
+Nautilus_Undo_Manager nautilus_undo_get_undo_manager    (GObject               *attached_object);
+void                  nautilus_undo_attach_undo_manager (GObject               *object,
+							 Nautilus_Undo_Manager  manager);
 
 #endif /* NAUTILUS_UNDO_PRIVATE_H */
